/*!
* Copyright Adaptavist 2023 (c) All rights reserved
*/
import { Response, BaseApiCore, Request, constructHeaders, executeHttpCall, constructUrl } from '@managed-api/commons-core';
import { errorMessageBuilder } from './common';
import { customErrorHandlers, errorStrategyBuilderConstructor } from './errorStrategy';
export class SalesforceApiCore extends BaseApiCore {
    constructor(options) {
        super();
        this.options = options;
        this.sObject = new SObjectGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.All = new AllGroup(this.sObject);
        if (!this.options?.errorStrategy && this.options?.errorStrategy !== null) {
            this.setGlobalErrorStrategy((builder) => builder.retryOnRateLimiting(-1));
        }
    }
    setGlobalErrorStrategy(errorStrategy) {
        this.options = {
            ...this.options ?? {},
            errorStrategy
        };
    }
    buildResponse(url, status, statusText, headers, body) {
        const response = new Response(url, status, statusText, headers, body);
        response.setBufferImplementation(this.implementation);
        return response;
    }
    getGlobalErrorStrategy() {
        return this.options?.errorStrategy;
    }
}
class AllGroup {
    constructor(sObject) {
        this.sObject = sObject;
    }
    /**
     * Gets basic metadata for a specified object, including some object properties, recent items, and URIs for other resources related to the object.
     * Alternative usage: sObject.getBasicInformation
     * Documentation: https://developer.salesforce.com/docs/atlas.en-us.242.0.api_rest.meta/api_rest/resources_sobject_basic_info_get.htm
     * @param options Request options.
    */
    async getSObjectBasicInformation(options) {
        return this.sObject.getBasicInformation(options);
    }
    /**
     * Gets a record based on the specified object and record ID.
     * Alternative usage: sObject.Rows.getRecord
     * Documentation: https://developer.salesforce.com/docs/atlas.en-us.242.0.api_rest.meta/api_rest/resources_sobject_retrieve_get.htm
     * @param options Request options.
    */
    async getSObjectRowRecord(options) {
        return this.sObject.Row.getRecord(options);
    }
    /**
     * Updates a record based on the specified object and record ID. Field values provided in the request body replace the existing values in the record.
     * Alternative usage: sObject.Rows.updateRecord
     * Documentation: https://developer.salesforce.com/docs/atlas.en-us.api_rest.meta/api_rest/resources_sobject_retrieve_patch.htm
     * @param options Request options.
    */
    async updateSObjectRowRecord(options) {
        return this.sObject.Row.updateRecord(options);
    }
    /**
     * Deletes records based on the specified object and record ID.
     * Alternative usage: sObject.Rows.deleteRecord
     * Documentation: https://developer.salesforce.com/docs/atlas.en-us.api_rest.meta/api_rest/resources_sobject_retrieve_delete.htm
     * @param options Request options.
    */
    async deleteSObjectRowRecord(options) {
        return this.sObject.Row.deleteRecord(options);
    }
}
class SObjectGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
        this.Row = new SObjectRowGroup(implementation, this.getGlobalErrorStrategy);
    }
    /**
     * Gets basic metadata for a specified object, including some object properties, recent items, and URIs for other resources related to the object.
     * Documentation: https://developer.salesforce.com/docs/atlas.en-us.242.0.api_rest.meta/api_rest/resources_sobject_basic_info_get.htm
     * @param options Request options.
     */
    async getBasicInformation(options) {
        const { headers, sObject, errorStrategy } = options ?? {};
        const request = new Request('GET', `/services/data/v57.0/sobjects/${encodeURIComponent(sObject)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class SObjectRowGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Gets a record based on the specified object and record ID.
     * Documentation: https://developer.salesforce.com/docs/atlas.en-us.242.0.api_rest.meta/api_rest/resources_sobject_retrieve_get.htm
     * @param options Request options.
     */
    async getRecord(options) {
        const { headers, sObject, id, fields, errorStrategy } = options ?? {};
        const queryParams = { fields };
        const request = new Request('GET', constructUrl(`/services/data/v57.0/sobjects/${encodeURIComponent(sObject)}/${encodeURIComponent(id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates a record based on the specified object and record ID. Field values provided in the request body replace the existing values in the record.
     * Documentation: https://developer.salesforce.com/docs/atlas.en-us.api_rest.meta/api_rest/resources_sobject_retrieve_patch.htm
     * @param options Request options.
     */
    async updateRecord(options) {
        const { headers, sObject, id, body, errorStrategy } = options ?? {};
        const request = new Request('PATCH', `/services/data/v57.0/sobjects/${encodeURIComponent(sObject)}/${encodeURIComponent(id)}`, constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
    * Deletes records based on the specified object and record ID.
    * Documentation: https://developer.salesforce.com/docs/atlas.en-us.api_rest.meta/api_rest/resources_sobject_retrieve_delete.htm
    * @param options Request options.
    */
    async deleteRecord(options) {
        const { headers, sObject, id, errorStrategy } = options ?? {};
        const request = new Request('DELETE', `/services/data/v57.0/sobjects/${encodeURIComponent(sObject)}/${encodeURIComponent(id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
}
